# -*- test-case-name: virtualbricks.tests.test_tuntaps -*-
# Virtualbricks - a vde/qemu gui written in python and GTK/Glade.
# Copyright (C) 2013 Virtualbricks team

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

import os

from virtualbricks import bricks, link, settings
import tools


if False:  # pyflakes
    _ = str


class CaptureConfig(bricks.Config):

    parameters = {"iface": bricks.String("")}


class Capture(bricks.Brick):

    type = "Capture"
    _needsudo = True
    config_factory = CaptureConfig
    command_builder = {"-s": None,
                       "*iface": "iface"}

    def __init__(self, factory, name):
        bricks.Brick.__init__(self, factory, name)
        self.plugs.append(link.Plug(self))
        self.command_builder["-s"] = self.sock_path

    def sock_path(self):
        if self.plugs[0].sock:
            return self.plugs[0].sock.path.rstrip("[]")
        return ""

    def get_parameters(self):
        if self.config["iface"] == "":
            return _("No interface selected")
        if self.plugs[0].sock:
            return _("Interface %s plugged to %s ") % (
                self.config["iface"], self.plugs[0].sock.brick.name)
        return _("Interface %s disconnected") % self.config["iface"]

    def prog(self):
        return os.path.join(settings.get("vdepath"), "vde_pcapplug")

    def open_console(self):
        pass

    def configured(self):
        return self.plugs[0].sock and self.config["iface"]

    def add_plug(self, sock, mac=None, model=None):
        # TODO: deduplicate code from VirtualMachine.add_plug
        plug = self.factory.new_plug(self)
        self.plugs.append(plug)
        if sock:
            plug.connect(sock)
        if mac:
            plug.mac = mac
        if model:
            plug.model = model
        return plug


class TapConfig(bricks.Config):

    parameters = {"ip": bricks.String("10.0.0.1"),
                  "nm": bricks.String("255.255.255.0"),
                  "gw": bricks.String(""),
                  "mode": bricks.String("off")}


class Tap(bricks.Brick):

    type = "Tap"
    _needsudo = True
    config_factory = TapConfig

    def __init__(self, factory, name):
        bricks.Brick.__init__(self, factory, name)
        self.plugs.append(link.Plug(self))
        self.command_builder["-s"] = self.sock_path
        self.command_builder["*tap"] = self.get_name

    def sock_path(self):
        if self.plugs[0].sock:
            return self.plugs[0].sock.path.rstrip("[]")
        return ""

    def get_parameters(self):
        if self.plugs[0].sock:
            return _("plugged to %s ") % self.plugs[0].sock.brick.name
        return _("disconnected")

    def prog(self):
        return os.path.join(settings.get("vdepath"), "vde_plug2tap")

    def open_console(self):
        pass

    def configured(self):
        return self.plugs[0].sock

    def post_poweron(self):
        self.start_related_events(on=True)
        if self.config["mode"] == 'dhcp':
            tools.configure_network_interface_dhcp(self.name)
        elif self.config["mode"] == 'manual':
            if self.config["gw"]:
                gateway = self.config["gw"][0]
            else:
                gateway = None
            tools.configure_network_interface_manually(self.name,
                    self.config["ip"], self.config["nm"], gateway=gateway)
        else:
            return
